# This code is loaded via `mach python --exec-file`, so it runs in the scope of
# the `mach python` command.
from __future__ import absolute_import
self._mach_context.commands.dispatch('uuid', self._mach_context)  # noqa: F821
